@@ -1,4 +1,5 @@
 from dataclasses import dataclass
+from functools import lru_cache
 from typing import (
     Any,
     Callable,
@@ -20,21 +21,24 @@ from apischema.conversions.utils import (
 )
 from apischema.dataclasses import replace
 from apischema.types import AnyType
-from apischema.utils import get_origin_or_type, is_method, method_class, method_wrapper
+from apischema.utils import deprecate_kwargs, is_method, method_class, method_wrapper
 
 if TYPE_CHECKING:
-    from apischema.deserialization.coercion import Coercion
+    from apischema.deserialization.coercion import Coerce
 
 
+@deprecate_kwargs({"sub_conversions": "sub_conversion"})
 @dataclass(frozen=True)
 class Conversion:
     converter: Union[Converter, property]
     source: AnyType = None
     target: AnyType = None
-    sub_conversions: Optional["Conversions"] = None
+    sub_conversion: Optional["AnyConversion"] = None
     additional_properties: Optional[bool] = None
-    coercion: Optional["Coercion"] = None
-    default_fallback: Optional[bool] = None
+    fall_back_on_any: Optional[bool] = None
+    check_type: Optional[bool] = None
+    coerce: Optional["Coerce"] = None
+    fall_back_on_default: Optional[bool] = None
     exclude_unset: Optional[bool] = None
     inherited: Optional[bool] = None
 
@@ -44,11 +48,21 @@ class Conversion:
 
 @dataclass(frozen=True)
 class LazyConversion:
-    get: Callable[[], Optional["Conversions"]]
+    get: Callable[[], Optional["AnyConversion"]]
+
+    def __post_init__(self):
+        object.__setattr__(self, "get", lru_cache(1)(self.get))
+
+    @property
+    def inherited(self) -> Optional[bool]:
+        conversion = self.get()  # type: ignore
+        return isinstance(conversion, Conversion) and conversion.inherited
 
 
 ConvOrFunc = Union[Conversion, Converter, property, LazyConversion]
-Conversions = Union[ConvOrFunc, Collection[ConvOrFunc]]
+AnyConversion = Union[ConvOrFunc, Tuple[ConvOrFunc, ...]]
+DefaultConversion = Callable[[AnyType], Optional[AnyConversion]]
+
 
 ResolvedConversion = NewType("ResolvedConversion", Conversion)
 ResolvedConversions = Tuple[ResolvedConversion, ...]  # Tuple in order to be hashable
@@ -70,51 +84,34 @@ def resolve_conversion(
     return ResolvedConversion(replace(conversion, source=source, target=target))
 
 
-def resolve_conversions(conversions: Optional[Conversions]) -> ResolvedConversions:
-    if not conversions:
+def resolve_any_conversion(conversion: Optional[AnyConversion]) -> ResolvedConversions:
+    if not conversion:
         return ()
     result: List[ResolvedConversion] = []
-    for conv in conversions if isinstance(conversions, Collection) else [conversions]:
+    for conv in conversion if isinstance(conversion, Collection) else [conversion]:
         if isinstance(conv, LazyConversion):
-            result.extend(resolve_conversions(conv.get()))  # type: ignore
+            result.extend(resolve_any_conversion(conv.get()))  # type: ignore
         else:
             result.append(resolve_conversion(conv))
     return tuple(result)
 
 
+def handle_identity_conversion(
+    conversion: ResolvedConversion, tp: AnyType
+) -> ResolvedConversion:
+    if is_identity(conversion) and conversion.source == IdentityT:
+        return ResolvedConversion(replace(conversion, source=tp, target=tp))
+    else:
+        return conversion
+
+
 def is_identity(conversion: ResolvedConversion) -> bool:
     return (
         conversion.converter == identity
-        and conversion.source == conversion.target == IdentityT
-        and conversion.sub_conversions is None
+        and conversion.source == conversion.target
+        and conversion.sub_conversion is None
         and conversion.additional_properties is None
-        and conversion.coercion is None
-        and conversion.default_fallback is None
+        and conversion.coerce is None
+        and conversion.fall_back_on_default is None
         and conversion.exclude_unset is None
     )
-
-
-def handle_container_conversions(
-    tp: AnyType,
-    next_conversions: Optional[Conversions],
-    prev_conversions: Optional[Conversions],
-    dynamic: bool,
-) -> Optional[Conversions]:
-    origin = get_origin_or_type(tp)
-    if (
-        prev_conversions
-        and not dynamic
-        and (
-            (isinstance(origin, type) and issubclass(origin, Collection))
-            or origin == Union
-        )
-    ):
-        if next_conversions:
-            return (
-                LazyConversion(lambda: next_conversions),
-                LazyConversion(lambda: prev_conversions),
-            )
-        else:
-            return prev_conversions
-    else:
-        return next_conversions
