@@ -1,13 +1,11 @@
 from collections import defaultdict
-from dataclasses import dataclass, is_dataclass
-from enum import Enum
-from functools import partial
-from inspect import Parameter, iscoroutinefunction, signature
+from dataclasses import dataclass
+from functools import lru_cache, wraps
+from inspect import Parameter, signature
 from typing import (
     Any,
     Awaitable,
     Callable,
-    Collection,
     Dict,
     Iterator,
     Mapping,
@@ -21,112 +19,91 @@ from typing import (
 
 import graphql
 
+from apischema import UndefinedType
 from apischema.aliases import Aliaser
-from apischema.conversions.conversions import Conversions, handle_container_conversions
-from apischema.conversions.dataclass_models import DataclassModel
-from apischema.deserialization import deserialize
-from apischema.json_schema.schema import Schema
-from apischema.metadata.implem import ConversionMetadata
-from apischema.metadata.keys import (
-    ALIAS_METADATA,
-    CONVERSIONS_METADATA,
-    DEFAULT_FALLBACK_METADATA,
-    REQUIRED_METADATA,
-    get_annotated_metadata,
-)
+from apischema.cache import cache
+from apischema.conversions.conversions import AnyConversion, DefaultConversion
+from apischema.deserialization import deserialization_method
+from apischema.objects import ObjectField
+from apischema.schemas import Schema
 from apischema.serialization import (
-    COLLECTION_TYPE_SET,
-    MAPPING_TYPE_SET,
-    PRIMITIVE_TYPES_SET,
-    get_conversions,
+    SerializationMethod,
+    SerializationMethodVisitor,
     serialize,
 )
 from apischema.serialization.serialized_methods import (
     ErrorHandler,
-    Serialized,
+    SerializedMethod,
     _get_methods,
     serialized as register_serialized,
 )
-from apischema.types import AnyType, NoneType, PRIMITIVE_TYPES
-from apischema.typing import get_origin, get_type_hints
+from apischema.types import AnyType, NoneType, Undefined
 from apischema.utils import (
-    Undefined,
+    awaitable_origin,
+    deprecate_kwargs,
+    empty_dict,
     get_args2,
-    get_origin_or_type,
+    get_origin_or_type2,
+    is_async,
     is_union_of,
+    keep_annotations,
     method_registerer,
 )
 from apischema.validation.errors import ValidationError
-from apischema.visitor import Unsupported
 
 
-def partial_serialize(
-    obj: Any, *, aliaser: Aliaser, conversions: Conversions = None
-) -> Any:
-    if obj is Undefined:
-        return None
-    cls = obj.__class__
-    if cls in PRIMITIVE_TYPES_SET:
-        return obj
-    if cls in COLLECTION_TYPE_SET:
-        return [
-            partial_serialize(elt, aliaser=aliaser, conversions=conversions)
-            for elt in obj
-        ]
-    if cls in MAPPING_TYPE_SET:
-        return serialize(
-            obj, conversions=conversions, aliaser=aliaser, exclude_unset=False
+class PartialSerializationMethodVisitor(SerializationMethodVisitor):
+    @property
+    def _any_method(self) -> Callable[[type], SerializationMethod]:
+        return partial_serialization_method_factory(
+            self.aliaser, self._conversions, self.default_conversion
         )
-    conversion, dynamic = get_conversions(cls, conversions)
-    if conversion is not None:
-        if isinstance(conversion.target, DataclassModel):
-            return obj
-        return partial_serialize(
-            conversion.converter(obj),  # type: ignore
-            aliaser=aliaser,
-            conversions=handle_container_conversions(
-                conversion.target,
-                conversion.sub_conversions,
-                conversions,
-                dynamic,
-            ),
-        )
-    if is_dataclass(cls):
-        return obj
-    if issubclass(cls, Enum):
-        return serialize(obj.value, aliaser=aliaser, exclude_unset=False)
-    if isinstance(obj, PRIMITIVE_TYPES):
-        return obj
-    if isinstance(obj, Mapping):
-        return serialize(obj, aliaser=aliaser, exclude_unset=False)
-    if isinstance(obj, Collection):
-        return [partial_serialize(elt, aliaser=aliaser) for elt in obj]
-    if issubclass(cls, tuple) and hasattr(cls, "_fields"):
-        return obj
-    raise Unsupported(cls)
+
+    def object(self, tp: AnyType, fields: Sequence[ObjectField]) -> SerializationMethod:
+        return lambda obj: obj
+
+    def union(self, alternatives: Sequence[AnyType]) -> SerializationMethod:
+        method = super().union([alt for alt in alternatives if alt != UndefinedType])
+        if UndefinedType in alternatives:
+
+            @wraps(method)
+            def wrapper(obj: Any) -> Any:
+                return obj if obj is Undefined else method(obj)
+
+            return wrapper
+        else:
+            return method
+
+    def unsupported(self, tp: AnyType) -> SerializationMethod:
+        if tp is UndefinedType:
+            return lambda obj: None
+        return super().unsupported(tp)
 
 
-awaitable_origin = get_origin(Awaitable[Any])
+@cache
+def partial_serialization_method_factory(
+    aliaser: Aliaser,
+    conversion: Optional[AnyConversion],
+    default_conversion: DefaultConversion,
+) -> Callable[[AnyType], SerializationMethod]:
+    @lru_cache()
+    def factory(tp: AnyType) -> SerializationMethod:
+        return PartialSerializationMethodVisitor(
+            aliaser, False, False, default_conversion, False, False
+        ).visit_with_conv(tp, conversion)
 
-
-def is_async(func: Callable) -> bool:
-    return (
-        iscoroutinefunction(func)
-        or get_origin_or_type(get_type_hints(func).get("return")) == awaitable_origin
-    )
+    return factory
 
 
 def unwrap_awaitable(tp: AnyType) -> AnyType:
-    if get_origin_or_type(tp) != awaitable_origin:
+    if get_origin_or_type2(tp) == awaitable_origin:
+        return keep_annotations(get_args2(tp)[0] if get_args2(tp) else Any, tp)
+    else:
         return tp
-    try:
-        return get_args2(tp)[0]
-    except IndexError:
-        return Any
 
 
 @dataclass(frozen=True)
-class Resolver(Serialized):
+class Resolver(SerializedMethod):
     parameters: Sequence[Parameter]
     parameters_metadata: Mapping[str, Mapping]
 
@@ -176,7 +153,7 @@ def resolver(__method_or_property: MethodOrProp) -> MethodOrProp:
 def resolver(
     alias: str = None,
     *,
-    conversions: Conversions = None,
+    conversion: AnyConversion = None,
     schema: Schema = None,
     error_handler: ErrorHandler = Undefined,
     parameters_metadata: Mapping[str, Mapping] = None,
@@ -186,20 +163,21 @@ def resolver(
     ...
 
 
+@deprecate_kwargs({"conversions": "conversion"})
 def resolver(
     __arg=None,
     *,
     alias: str = None,
-    conversions: Conversions = None,
+    conversion: AnyConversion = None,
     schema: Schema = None,
     error_handler: ErrorHandler = Undefined,
     parameters_metadata: Mapping[str, Mapping] = None,
     serialized: bool = False,
     owner: Type = None,
 ):
-    def register(func: Callable, owner: Optional[Type], alias2: str):
+    def register(func: Callable, owner: Type, alias2: str):
         alias2 = alias or alias2
-        first_param, *parameters = resolver_parameters(func, check_first=owner is None)
+        _, *parameters = resolver_parameters(func, check_first=owner is None)
         error_handler2 = error_handler
         if error_handler2 is None:
             error_handler2 = none_error_handler
@@ -207,17 +185,12 @@ def resolver(
             error_handler2 = None
         resolver = Resolver(
             func,
-            conversions,
+            conversion,
             schema,
             error_handler2,
             parameters,
             parameters_metadata or {},
         )
-        if owner is None:
-            try:
-                owner = get_origin_or_type(get_type_hints(func)[first_param.name])
-            except KeyError:
-                raise TypeError("First parameter of resolver must be typed") from None
         _resolvers[owner][alias2] = resolver
         if serialized:
             if is_async(func):
@@ -225,7 +198,7 @@ def resolver(
             try:
                 register_serialized(
                     alias=alias2,
-                    conversions=conversions,
+                    conversion=conversion,
                     schema=schema,
                     error_handler=error_handler,
                     owner=owner,
@@ -239,10 +212,23 @@ def resolver(
     return method_registerer(__arg, owner, register)
 
 
+T = TypeVar("T")
+U = TypeVar("U")
+
+
+def as_async(func: Callable[[T], U]) -> Callable[[Awaitable[T]], Awaitable[U]]:
+    async def wrapper(arg: Awaitable[T]) -> U:
+        return func(await arg)
+
+    return wrapper
+
+
 def resolver_resolve(
     resolver: Resolver,
     types: Mapping[str, AnyType],
     aliaser: Aliaser,
+    default_deserialization: DefaultConversion,
+    default_serialization: DefaultConversion,
     serialized: bool = True,
 ) -> Callable:
     parameters, info_parameter = [], None
@@ -251,60 +237,63 @@ def resolver_resolve(
         if is_union_of(param_type, graphql.GraphQLResolveInfo):
             info_parameter = param.name
         else:
-            metadata = get_annotated_metadata(param_type)
-            if param.name in resolver.parameters_metadata:
-                metadata = {**metadata, **resolver.parameters_metadata[param.name]}
-            alias = metadata.get(ALIAS_METADATA, param.name)
-            deserializer = partial(
-                deserialize,
+            param_field = ObjectField(
+                param.name,
                 param_type,
-                conversions=metadata.get(
-                    CONVERSIONS_METADATA, ConversionMetadata()
-                ).deserialization,
-                aliaser=aliaser,
-                default_fallback=DEFAULT_FALLBACK_METADATA in metadata or None,
+                param.default is Parameter.empty,
+                resolver.parameters_metadata.get(param.name, empty_dict),
+                param.default,
             )
-            required = REQUIRED_METADATA in metadata or param.default is Parameter.empty
-            opt_param = is_union_of(param_type, NoneType)
+            deserializer = deserialization_method(
+                param_type,
+                additional_properties=False,
+                aliaser=aliaser,
+                coerce=False,
+                conversion=param_field.deserialization,
+                default_conversion=default_deserialization,
+                fall_back_on_default=False,
+                schema=param_field.schema,
+            )
+            opt_param = is_union_of(param_type, NoneType) or param.default is None
             parameters.append(
-                (aliaser(alias), param.name, deserializer, opt_param, required)
+                (
+                    aliaser(param_field.alias),
+                    param.name,
+                    deserializer,
+                    opt_param,
+                    param_field.required,
+                )
             )
     func, error_handler = resolver.func, resolver.error_handler
-
-    def no_serialize(result):
-        return result
-
-    async def async_serialize(result: Awaitable):
-        return partial_serialize(
-            await result, aliaser=aliaser, conversions=resolver.conversions
-        )
-
-    def sync_serialize(result):
-        return partial_serialize(
-            result, aliaser=aliaser, conversions=resolver.conversions
-        )
+    method_factory = partial_serialization_method_factory(
+        aliaser, resolver.conversion, default_serialization
+    )
 
     serialize_result: Callable[[Any], Any]
     if not serialized:
-        serialize_result = no_serialize
+        serialize_result = lambda res: res
     elif is_async(resolver.func):
-        serialize_result = async_serialize
+        serialize_result = as_async(method_factory(types["return"]))
     else:
-        serialize_result = sync_serialize
+        serialize_result = method_factory(types["return"])
     serialize_error: Optional[Callable[[Any], Any]]
     if error_handler is None:
         serialize_error = None
     elif is_async(error_handler):
-        serialize_error = async_serialize
+        serialize_error = as_async(method_factory(resolver.error_type()))
     else:
-        serialize_error = sync_serialize
+        serialize_error = method_factory(resolver.error_type())
 
     def resolve(__self, __info, **kwargs):
         values = {}
         errors: Dict[str, ValidationError] = {}
         for alias, param_name, deserializer, opt_param, required in parameters:
             if alias in kwargs:
-                if not opt_param and kwargs[param_name] is None:
+                # It is possible for the parameter to be non-optional in Python
+                # type hints but optional in the generated schema. In this case
+                # we should ignore it.
+                # See: https://github.com/wyfo/apischema/pull/130#issuecomment-845497392
+                if not opt_param and kwargs[alias] is None:
                     assert not required
                     continue
                 try:
@@ -315,7 +304,11 @@ def resolver_resolve(
                 values[param_name] = None
 
         if errors:
-            raise TypeError(serialize(ValidationError(children=errors)))
+            raise TypeError(
+                serialize(
+                    ValidationError, ValidationError(children=errors), aliaser=aliaser
+                )
+            )
         if info_parameter:
             values[info_parameter] = __info
         try:
