@@ -1,8 +1,8 @@
 """Kind of typing_extensions for this package"""
-__all__ = ["_LiteralMeta", "_TypedDictMeta", "get_args", "get_origin", "get_type_hints"]
+__all__ = ["get_args", "get_origin", "get_type_hints"]
 
 import sys
-from types import ModuleType
+from types import ModuleType, new_class
 from typing import (  # type: ignore
     Any,
     Callable,
@@ -68,7 +68,7 @@ else:  # pragma: no cover
             if hasattr(tp, "_subs_tree"):
                 tp = _assemble_tree(tp._subs_tree())
             if isinstance(tp, _AnnotatedAlias):
-                return Annotated
+                return None if tp.__args__ is None else Annotated
             if tp is Generic:
                 return Generic
             return getattr(tp, "__origin__", None)
@@ -78,7 +78,7 @@ else:  # pragma: no cover
             if hasattr(tp, "_subs_tree"):
                 tp = _assemble_tree(tp._subs_tree())
             if isinstance(tp, _AnnotatedAlias):
-                return (tp.__args__[0], *tp.__metadata__)
+                return () if tp.__args__ is None else (tp.__args__[0], *tp.__metadata__)
             # __args__ can be None in 3.6 inside __set_name__
             res = getattr(tp, "__args__", ()) or ()
             if get_origin(tp) is Callable and res[0] is not Ellipsis:
@@ -209,9 +209,63 @@ except NameError:
     _LiteralMeta, _TypedDictMeta = _FakeType, _FakeType  # type: ignore
 
 
+def is_new_type(tp: Any) -> bool:
+    return hasattr(tp, "__supertype__")
+
+
+def is_annotated(tp: Any) -> bool:
+    try:
+        from typing import Annotated  # type: ignore
+
+        return get_origin(tp) == Annotated
+    except ImportError:
+        try:
+            from typing_extensions import Annotated  # type: ignore
+
+            return get_origin(tp) == Annotated
+        except ImportError:
+            return False
+
+
+def is_literal(tp: Any) -> bool:
+    try:
+        from typing import Literal
+
+        return get_origin(tp) == Literal or isinstance(tp, type(Literal))  # py36
+    except ImportError:
+        try:
+            from typing_extensions import Literal  # type: ignore
+
+            return get_origin(tp) == Literal or isinstance(tp, type(Literal))  # py36
+        except ImportError:
+            return False
+
+
+def is_named_tuple(tp: Any) -> bool:
+    return issubclass(tp, tuple) and hasattr(tp, "_fields")
+
+
+def is_typed_dict(tp: Any) -> bool:
+    try:
+        from typing import TypedDict
+
+        return isinstance(tp, type(new_class("_TypedDictImplem", (TypedDict,))))
+    except ImportError:
+        try:
+            from typing_extensions import TypedDict  # type: ignore
+
+            return isinstance(tp, type(new_class("_TypedDictImplem", (TypedDict,))))
+        except ImportError:
+            return False
+
+
+def is_type_var(tp: Any) -> bool:
+    return isinstance(tp, TypeVar)  # type: ignore
+
+
 # Don't use sys.version_info because it can also depend of typing_extensions version
 def required_keys(typed_dict: Type) -> Collection[str]:
-    assert isinstance(typed_dict, _TypedDictMeta)
+    assert is_typed_dict(typed_dict)
     if hasattr(typed_dict, "__required_keys__"):
         return typed_dict.__required_keys__
     else:
@@ -225,3 +279,25 @@ def required_keys(typed_dict: Type) -> Collection[str]:
         if typed_dict.__total__:  # type: ignore
             required.update(typed_dict.__annotations__.keys() - bases_annotations)
         return required
+
+
+# py37/py38 get_origin of builtin wrapped generics return the unsubscriptable builtin
+# type.
+if (3, 7) <= sys.version_info < (3, 9):
+    import typing
+
+    TYPING_ALIASES = {
+        getattr(elt, "__origin__", None): elt for elt in typing.__dict__.values()
+    }
+
+    def typing_origin(origin: Any) -> Any:
+        return TYPING_ALIASES.get(origin, origin)
+
+
+else:
+    typing_origin = lambda tp: tp
+
+
+def is_type(tp: Any) -> bool:
+    """isinstance is not enough because in py39: isinstance(list[int], type) == True"""
+    return isinstance(tp, type) and not get_args(tp)
